"""Plugin system for webhook_bridge.

This module provides the base plugin class and utilities for loading
and executing webhook plugins. It maintains compatibility with v0.6.0.
"""

# Import built-in modules

# Import built-in modules
import importlib.util
from pathlib import Path
import traceback
from typing import Any
from typing import Dict
from typing import Optional


class BasePlugin:
    """Abstract base class for all webhook bridge plugins.

    This class defines the interface that all webhook bridge plugins must implement.
    For backward compatibility, both `run` and `handle` methods are supported.

    Attributes:
        data: Dictionary containing the plugin's input data
        logger: Logger instance for the plugin
        http_method: The HTTP method used to call the plugin
    """

    def __init__(self, data: Dict[str, Any], logger: Optional[Any] = None, http_method: str = "POST"):
        """Initialize the plugin with data and logger."""
        self.data = data
        if logger is None:
            # Import built-in modules
            import logging
            logger = logging.getLogger(self.__class__.__name__)
        self.logger = logger
        self.http_method = http_method.upper()
        self.result = {}

    def run(self) -> Dict[str, Any]:
        """Execute the plugin's main functionality (deprecated but supported)."""
        # For backward compatibility, check if subclass overrides run
        if hasattr(self.__class__, "run") and self.__class__.run is not BasePlugin.run:
            # Call the subclass implementation
            return self._call_subclass_run()

        # Otherwise, dispatch to appropriate method handler
        if self.http_method == "GET" and hasattr(self, "get"):
            return self.get()
        elif self.http_method == "POST" and hasattr(self, "post"):
            return self.post()
        elif self.http_method == "PUT" and hasattr(self, "put"):
            return self.put()
        elif self.http_method == "DELETE" and hasattr(self, "delete"):
            return self.delete()
        else:
            return self.handle()

    def _call_subclass_run(self) -> Dict[str, Any]:
        """Call the subclass implementation of run method."""
        # This is a bit tricky - we need to call the actual subclass method
        # For now, return a default implementation
        return {"message": "Plugin executed successfully", "data": self.data}

    def handle(self) -> Dict[str, Any]:
        """Generic handler for the plugin's functionality."""
        return {"message": "Plugin executed successfully", "data": self.data}

    def get(self) -> Dict[str, Any]:
        """Handle GET requests."""
        return self.handle()

    def post(self) -> Dict[str, Any]:
        """Handle POST requests."""
        return self.handle()

    def put(self) -> Dict[str, Any]:
        """Handle PUT requests."""
        return self.handle()

    def delete(self) -> Dict[str, Any]:
        """Handle DELETE requests."""
        return self.handle()

    def execute(self) -> Dict[str, Any]:
        """Execute the plugin and format the results (v0.6.0 compatible)."""
        try:
            result = self.run()
            if not isinstance(result, dict):
                result = {"result": result}

            # Format in v0.6.0 compatible format
            return {
                "input_data": self.data,
                "additional_info": "This is some additional information.",
                "http_method": self.http_method,
                "result": result,
                "plugin_executed": True,
                "status": "success",
            }
        except Exception as e:
            return {
                "input_data": self.data,
                "plugin_executed": False,
                "error": str(e),
                "traceback": traceback.format_exc(),
                "http_method": self.http_method,
                "status": "error",
            }

    def get_data(self, key: str, default: Any = None) -> Any:
        """Get data from the webhook payload."""
        return self.data.get(key, default)

    def set_result(self, key: str, value: Any) -> None:
        """Set a result value."""
        self.result[key] = value

    def get_result(self) -> Dict[str, Any]:
        """Get the current result dictionary."""
        return self.result.copy()


def load_plugin(plugin_path: str) -> type:
    """Load a plugin from a file path."""
    try:
        name = Path(plugin_path).stem
        spec = importlib.util.spec_from_file_location(name, plugin_path)
        if spec is None or spec.loader is None:
            raise ImportError(f"Cannot create module spec for {plugin_path}")

        module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(module)

        # Find the plugin class
        plugin_class = None
        for attr_name in dir(module):
            attr = getattr(module, attr_name)
            if (isinstance(attr, type) and
                issubclass(attr, BasePlugin) and
                attr is not BasePlugin):
                plugin_class = attr
                break

        if plugin_class is None:
            raise AttributeError(f"No plugin class found in {plugin_path}")

        return plugin_class

    except Exception as e:
        raise ImportError(f"Failed to load plugin from {plugin_path}: {e}")


def validate_plugin_class(plugin_class: type) -> bool:
    """Validate that a class is a proper plugin."""
    try:
        return (
            isinstance(plugin_class, type) and
            issubclass(plugin_class, BasePlugin) and
            plugin_class is not BasePlugin
        )
    except Exception:
        return False


class PluginManager:
    """Manager for loading and executing plugins."""

    def __init__(self):
        self._plugin_cache = {}

    def load_plugin(self, plugin_path: str, use_cache: bool = True) -> type:
        """Load a plugin with optional caching."""
        if use_cache and plugin_path in self._plugin_cache:
            return self._plugin_cache[plugin_path]

        plugin_class = load_plugin(plugin_path)

        if use_cache:
            self._plugin_cache[plugin_path] = plugin_class

        return plugin_class

    def clear_cache(self) -> None:
        """Clear the plugin cache."""
        self._plugin_cache.clear()

    def execute_plugin(self, plugin_path: str, data: Dict[str, Any],
                      http_method: str = "POST") -> Dict[str, Any]:
        """Execute a plugin."""
        plugin_class = self.load_plugin(plugin_path)
        plugin_instance = plugin_class(data, None, http_method)
        return plugin_instance.execute()
